@@ -23,7 +23,7 @@
 
 
 #include "rhelp.h"
-/* #include "rand_draws.h" */
+
 #include "matrix.h"
 #include <assert.h>
 #include <math.h>
@@ -139,13 +139,74 @@ double ** new_matrix_bones(double *v, unsigned int n1, unsigned int n2)
 {
   double **M;
   int i;
-  M = (double **)  malloc(sizeof(double*) * n1);
+  M = (double **) malloc(sizeof(double*) * n1);
   M[0] = v;
   for(i=1; i<n1; i++) M[i] = M[i-1] + n2;
   return(M);
 }
 
 
+/*
+ * create an integer ** Matrix from a double * vector
+ * should be freed with the free command, rather than
+ * delete_matrix
+ */
+
+int ** new_imatrix_bones(int *v, unsigned int n1, unsigned int n2)
+{
+  int **M;
+  int i;
+  M = (int **) malloc(sizeof(double*) * n1);
+  M[0] = v;
+  for(i=1; i<n1; i++) M[i] = M[i-1] + n2;
+  return(M);
+}
+
+
+/*
+ * create a new n1 x n2 integer matrix which is allocated like
+ * and n1*n2 array, but can be referenced as a 2-d array
+ */
+
+int ** new_imatrix(unsigned int n1, unsigned int n2)
+{
+  int i;
+  int **m;
+  
+  if(n1 == 0 || n2 == 0) return NULL;
+  
+  m = (int**) malloc(sizeof(double*) * n1);
+  assert(m);
+  m[0] = (int*) malloc(sizeof(double) * (n1*n2));
+  assert(m[0]);
+  
+  for(i=1; i<n1; i++) m[i] = m[i-1] + n2;
+  
+  return m;
+}
+
+
+/*
+ * create a new n2 x n1 integer matrix which is allocated like
+ * and n1*n2 array, and copy the TRANSPOSE of n1 x n2 M into it.
+ */
+
+int ** new_t_imatrix(int** M, unsigned int n1, unsigned int n2)
+{
+  int i,j;
+  int **m;
+  
+  if(n1 <= 0 || n2 <= 0) {
+    assert(M == NULL);
+    return NULL;
+  }
+  
+  m = new_imatrix(n2, n1);
+  for(i=0; i<n1; i++) for(j=0; j<n2; j++)  m[j][i] = M[i][j];
+  return m;
+}
+
+
 /*
  * create a new n1 x n2 matrix which is allocated like
  * and n1*n2 array, and copy the of n1 x n2 M into it.
@@ -309,8 +370,9 @@ void delete_matrix(double** m)
   free(m);
 }
 
+
 /*
- * delete a matrix allocated as above
+ * delete an integer matrix allocated as above
  */
 
 void delete_imatrix(int** m)
@@ -1317,17 +1379,18 @@ void matrix_t_to_file(const char* file_str, double** matrix, unsigned int n1,
  */
 
 void sub_p_matrix(double **V, int *p, double **v, 
-		unsigned int nrows, unsigned int lenp)
+		  unsigned int nrows, unsigned int lenp, 
+		  unsigned int col_offset)
 {
   int i,j;
   assert(V); assert(p); assert(v); assert(nrows > 0 && lenp > 0);
   for(i=0; i<nrows; i++) for(j=0; j<lenp; j++) 
-    V[i][j] = v[i][p[j]];
+    V[i][j+col_offset] = v[i][p[j]];
 }
 
 
 /*
- * new_p_matrix::
+ * new_p_submatrix:
  *
  * create a new matrix from the columns of v, specified
  * by p.  Must have have nrow(v) == nrow(V) and ncol(V) >= ncols
@@ -1335,13 +1398,12 @@ void sub_p_matrix(double **V, int *p, double **v,
  */
 
 double **new_p_submatrix(int *p, double **v, unsigned int nrows, 
-			 unsigned int ncols)
+			 unsigned int ncols, unsigned int col_offset)
 {
   double **V;
-  if(nrows == 0 || ncols == 0) return NULL;
-  assert(p); assert(v);
-  V = new_matrix(nrows, ncols);
-  sub_p_matrix(V, p, v, nrows, ncols);
+  if(nrows == 0 || ncols+col_offset == 0) return NULL;
+  V = new_matrix(nrows, ncols + col_offset);
+  if(ncols > 0) sub_p_matrix(V, p, v, nrows, ncols, col_offset);
   return(V);
 }
 
@@ -1552,7 +1614,9 @@ void normalize(double **X, double **rect, int N, int d, double normscale)
       else
 	X[j][i] = (X[j][i] - rect[0][i]) / norm;
       X[j][i] = normscale * X[j][i];
-      assert(X[j][i] >=0 && X[j][i] <= normscale);
+      /* if(!(X[j][i] >=0 && X[j][i] <= normscale))
+	myprintf(stdout, "X[%d][%d] = %g, normscale = %g\n", j, i, X[j][i], normscale);
+	assert(X[j][i] >=0 && X[j][i] <= normscale); */
     }
   }
 }
